@@ -19,8 +19,8 @@
  * \note
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA  02110-1301  USA
  */
 
 #ifndef __GPHOTO2_CAMERA_H__
@@ -136,6 +136,7 @@ typedef enum {
  * \returns a gphoto error code 
  */
 typedef int (*CameraExitFunc)      (Camera *camera, GPContext *context);
+
 /**
  * \brief Get a configuration tree for the camera and its driver
  *
@@ -157,6 +158,55 @@ typedef int (*CameraExitFunc)      (Camera *camera, GPContext *context);
  */
 typedef int (*CameraGetConfigFunc) (Camera *camera, CameraWidget **widget,
 				    GPContext *context);
+/**
+ * \brief Get a configuration widget for a specific configuration
+ *
+ * \param camera the current camera
+ * \param name the name of the widget
+ * \param widget pointer to store the toplevel widget of the tree
+ * \param context the active #GPContext
+ *
+ * A camera driver can support configuration of either its own behaviour
+ * or the camera device itself. To allow a flexible driver framework,
+ * the camera driver provides a generic configuration widget tree to the
+ * frontend, which then renders it, allows user input and sends it back
+ * via the #CameraSetConfigFunc function to have the driver configure itself
+ * or the camera.
+ * 
+ * This specific function retrieves one specific named entry, and not the full
+ * tree to allow for querying specific settings faster.
+ *
+ * If you do not have configuration ability, there is no need to specify this
+ * function.
+ *
+ * \returns a gphoto error code
+ */
+typedef int (*CameraGetSingleConfigFunc) (Camera *camera, const char *name, CameraWidget **widget,
+				    GPContext *context);
+/**
+ * \brief List all configuration widgets for a specific configuration
+ *
+ * \param camera the current camera
+ * \param list the list of widgets available
+ * \param context the active #GPContext
+ *
+ * A camera driver can support configuration of either its own behaviour
+ * or the camera device itself. To allow a flexible driver framework,
+ * the camera driver provides a generic configuration widget tree to the
+ * frontend, which then renders it, allows user input and sends it back
+ * via the #CameraSetConfigFunc function to have the driver configure itself
+ * or the camera.
+ * 
+ * This specific function retrieves all the available configuration values in a flat list.
+ *
+ * This is different than the GetConfigFunc, which returns a configuration tree.
+ *
+ * If you do not have configuration ability, there is no need to specify this
+ * function.
+ *
+ * \returns a gphoto error code
+ */
+typedef int (*CameraListConfigFunc) (Camera *camera, CameraList *list, GPContext *context);
 /**
  * \brief Set the configuration in the camera
  *
@@ -172,8 +222,24 @@ typedef int (*CameraGetConfigFunc) (Camera *camera, CameraWidget **widget,
  */
 typedef int (*CameraSetConfigFunc) (Camera *camera, CameraWidget  *widget,
 				    GPContext *context);
+/**
+ * \brief Set a single configuration variable in the camera
+ *
+ * \param camera the current camera 
+ * \param name the widget to set
+ * \param widget the configuration widget tree that was changed
+ * \param context the active #GPContext
+ *
+ * This function is called in the driver after the configuration value is set.
+ * 
+ * \returns a gphoto error code
+ */
+typedef int (*CameraSetSingleConfigFunc) (Camera *camera, const char *name, CameraWidget  *widget,
+				    GPContext *context);
+
 typedef int (*CameraCaptureFunc)   (Camera *camera, CameraCaptureType type,
 				    CameraFilePath *path, GPContext *context);
+typedef int (*CameraTriggerCaptureFunc)   (Camera *camera, GPContext *context);
 typedef int (*CameraCapturePreviewFunc) (Camera *camera, CameraFile *file,
 					 GPContext *context);
 typedef int (*CameraSummaryFunc)   (Camera *camera, CameraText *text,
@@ -226,8 +292,13 @@ typedef struct _CameraFunctions {
 	CameraGetConfigFunc       get_config;	/**< \brief Called for requesting the configuration widgets. */
 	CameraSetConfigFunc       set_config;	/**< \brief Called after a configuration was changed */
 
+	CameraListConfigFunc      list_config;	/**< \brief Called for listing the available configuration widgets. */
+	CameraGetSingleConfigFunc get_single_config;	/**< \brief Called for requesteing a single widget. */
+	CameraSetSingleConfigFunc set_single_config;	/**< \brief Called for setting a single configuration widget. */
+
 	/* Capturing */
 	CameraCaptureFunc        capture;	/**< \brief Remote control the camera to capture */
+	CameraTriggerCaptureFunc trigger_capture;/**< \brief Remote control the camera to trigger capture */
 	CameraCapturePreviewFunc capture_preview;/**< \brief Preview viewfinder content. */
 
 	/* Textual information */
@@ -237,7 +308,6 @@ typedef struct _CameraFunctions {
 
 	/* Event Interface */
 	CameraWaitForEvent wait_for_event;	/**< \brief Wait for a specific event from the camera */
-
 	/* Reserved space to use in the future without changing the struct size */
 	void *reserved1;			/**< \brief reserved for future use */
 	void *reserved2;			/**< \brief reserved for future use */
@@ -246,12 +316,9 @@ typedef struct _CameraFunctions {
 	void *reserved5;			/**< \brief reserved for future use */
 	void *reserved6;			/**< \brief reserved for future use */
 	void *reserved7;			/**< \brief reserved for future use */
+	void *reserved8;			/**< \brief reserved for future use */
 } CameraFunctions;
 
-/* Those are DEPRECATED */
-typedef GPPort     CameraPort;
-typedef GPPortInfo CameraPortInfo;
-
 typedef struct _CameraPrivateLibrary  CameraPrivateLibrary;
 typedef struct _CameraPrivateCore     CameraPrivateCore;
 
@@ -301,6 +368,7 @@ int gp_camera_get_port_speed    (Camera *camera);
 /** \name Initialization 
  * @{ 
  */
+int gp_camera_autodetect 	 (CameraList *list, GPContext *context);
 int gp_camera_init               (Camera *camera, GPContext *context);
 int gp_camera_exit               (Camera *camera, GPContext *context);
 
@@ -317,8 +385,14 @@ int gp_camera_free 		 (Camera *camera);
 
 int gp_camera_get_config	 (Camera *camera, CameraWidget **window,
 				  GPContext *context);
+int gp_camera_list_config	 (Camera *camera, CameraList *list,
+				  GPContext *context);
+int gp_camera_get_single_config	 (Camera *camera, const char *name, CameraWidget **widget,
+				  GPContext *context);
 int gp_camera_set_config	 (Camera *camera, CameraWidget  *window,
 				  GPContext *context);
+int gp_camera_set_single_config	 (Camera *camera, const char *name, CameraWidget  *widget,
+				  GPContext *context);
 int gp_camera_get_summary	 (Camera *camera, CameraText *summary,
 				  GPContext *context);
 int gp_camera_get_manual	 (Camera *camera, CameraText *manual,
@@ -327,6 +401,7 @@ int gp_camera_get_about		 (Camera *camera, CameraText *about,
 				  GPContext *context);
 int gp_camera_capture 		 (Camera *camera, CameraCaptureType type,
 				  CameraFilePath *path, GPContext *context);
+int gp_camera_trigger_capture 	 (Camera *camera, GPContext *context);
 int gp_camera_capture_preview 	 (Camera *camera, CameraFile *file,
 				  GPContext *context);
 int gp_camera_wait_for_event     (Camera *camera, int timeout,
@@ -348,7 +423,9 @@ int gp_camera_folder_list_folders (Camera *camera, const char *folder,
 				   CameraList *list, GPContext *context);
 int gp_camera_folder_delete_all   (Camera *camera, const char *folder,
 				   GPContext *context);
-int gp_camera_folder_put_file     (Camera *camera, const char *folder,
+int gp_camera_folder_put_file     (Camera *camera,
+				   const char *folder, const char *filename,
+				   CameraFileType type,
 				   CameraFile *file, GPContext *context);
 int gp_camera_folder_make_dir     (Camera *camera, const char *folder,
 				   const char *name, GPContext *context);
@@ -369,6 +446,10 @@ int gp_camera_file_set_info 	(Camera *camera, const char *folder,
 int gp_camera_file_get		(Camera *camera, const char *folder, 
 				 const char *file, CameraFileType type,
 				 CameraFile *camera_file, GPContext *context);
+int gp_camera_file_read		(Camera *camera, const char *folder, const char *file,
+		    		 CameraFileType type, 
+		    		 uint64_t offset, char *buf, uint64_t *size,
+		    		 GPContext *context);
 int gp_camera_file_delete     	(Camera *camera, const char *folder, 
 				 const char *file, GPContext *context);
 /**@}*/
@@ -399,4 +480,5 @@ void         gp_camera_stop_timeout      (Camera *camera, unsigned int id);
 }
 #endif /* __cplusplus */
 
+
 #endif /* __GPHOTO2_CAMERA_H__ */
